<?php
namespace Controller;

use \Service\Manager\Mongo\Manager;
use Service\DeleteCascadeService;
use Util\Utilities;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Util\Environment;

class DataSourceDependencies extends Crud {
    
    protected $dataSourceName = 'dependentDataSources';
    protected $organizationId;
    
    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, Environment $environment) {
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->organizationId = $environment->getOrganizationId();
    }
    
    public function getDataSourceDependencies(DTO\Request\Filter $request, DTO\Response $response){
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->setDataSourceName($this->dataSourceName);
        $rows = $this->mongoManager->findBy($filterCriteria)->getRows();
        
        $response->addDataSet(new DataSet($this->dataSourceName, $rows));
    }
    
    public function updateDependentDataSources(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $joinDataSourceId = $row['joinDataSourceId'];
        $persistedRows = [];
        
        $persistedRows['Dependencies removed'] = $this->deleteDependencies($row['oldOriginDataSources'], $joinDataSourceId);
        $persistedRows['Dependencies added'] = $this->addDependencies($row['newOriginDataSources'], $joinDataSourceId);
        
        $response->setDataSets(array(new DataSet($this->dataSourceName, $persistedRows)));
    }
    
    protected function deleteDependencies($originDataSourceIds = array(), $joinDataSourceId){
        $updatedDependencies = array();
        
        foreach ($originDataSourceIds as $originDataSourceId) {
            $dependency = $this->findDataSourceDependencies($originDataSourceId)->getRows();
            if($dependency){
                $dependency = current($dependency);

                $dependency = Utilities::objectToArray($dependency);
                $dependentDataSources = &$dependency['dependentDataSources'];
                $key = array_search($joinDataSourceId, $dependentDataSources);
                
                if($key !== false){
                    unset($dependentDataSources[$key]);
                    
                    if(!count($dependentDataSources)){
                        $this->deleteDependencyFile($dependency);
                    }
                }
                
                array_push($updatedDependencies, $dependency);
            }
        }
        
        $dataSet = new DataSet($this->dataSourceName, $updatedDependencies);
        return $this->mongoManager->persist($dataSet);
    }
    
    protected function addDependencies($originDataSourceIds = array(), $joinDataSourceId){
        $updatedDependencies = array();
        
        foreach ($originDataSourceIds as $originDataSourceId) {
            $dependency = $this->findDataSourceDependencies($originDataSourceId)->getRows();
            if($dependency){
                $dependency = $dependency[0];
                $dependency = Utilities::objectToArray($dependency);
                $dependentDataSources = &$dependency['dependentDataSources'];
                $key = array_search($joinDataSourceId, $dependentDataSources);
                
                if($key === false){
                    array_push($dependentDataSources, $joinDataSourceId);
                }
                
                array_push($updatedDependencies, $dependency);
            } else {
                $dependency = [
                    "originDataSourceId"   => $originDataSourceId,
                    "dependentDataSources" => array($joinDataSourceId),
                    "organizationId"       => $this->organizationId
                ];
                
                array_push($updatedDependencies, $dependency);
            }
        }
        
        $dataSet = new DataSet($this->dataSourceName, $updatedDependencies);
        return $this->mongoManager->persist($dataSet); 
    }
    
    protected function findDataSourceDependencies($originDataSourceId){
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('originDataSourceId', $originDataSourceId);
        $filterCriteria->addCondition('organizationId',  $this->organizationId);
        
        return $this->mongoManager->findBy($filterCriteria);
    }
    
    protected function deleteDependencyFile($dependency){
        $dataSet = new DataSet($this->dataSourceName, array($dependency));
        return $this->mongoManager->delete($dataSet);
    }
}

